@@ -4,7 +4,7 @@ use std::collections::HashMap;
 
 /// A structure that represents a tag element.
 /// grammar: `<[/]name [attr[="value"]] [/]>`
-#[derive(Debug, Clone)]
+#[derive(Debug, PartialEq, Clone)]
 pub struct Tag {
     name: String,
     attrs: Option<HashMap<String, String>>,
@@ -17,9 +17,9 @@ impl Tag {
     ///
     /// # Arguments
     /// * `name` - If `<h1 class="section1">`, then `h1`.
-    pub fn new(name: String) -> Tag {
+    pub fn new(name: &str) -> Tag {
         Tag {
-            name,
+            name: String::from(name),
             attrs: None,
             terminated: false,
             terminator: false,
@@ -43,7 +43,7 @@ impl Tag {
     /// use std::collections::HashMap;
     /// use parsercher::dom::Tag;
     ///
-    /// let mut tag = Tag::new("h1".to_string());
+    /// let mut tag = Tag::new("h1");
     /// let mut attrs: HashMap<String, String> = HashMap::new();
     /// attrs.insert("id".to_string(), "title".to_string());
     /// attrs.insert("class".to_string(), "section1".to_string());
@@ -61,7 +61,7 @@ impl Tag {
     /// ```
     /// use parsercher::dom::Tag;
     ///
-    /// let mut tag = Tag::new("h1".to_string());
+    /// let mut tag = Tag::new("h1");
     /// tag.set_attr("id", "title");
     /// tag.set_attr("class", "section1");
     ///
@@ -81,7 +81,7 @@ impl Tag {
     /// ```
     /// use parsercher::dom::Tag;
     ///
-    /// let mut tag = Tag::new("h1".to_string());
+    /// let mut tag = Tag::new("h1");
     /// tag.set_attr("id", "title");
     /// tag.set_attr("class", "section1");
     /// ```
@@ -105,7 +105,7 @@ impl Tag {
     /// ```
     /// use parsercher::dom::Tag;
     ///
-    /// let mut tag = Tag::new("h1".to_string());
+    /// let mut tag = Tag::new("h1");
     /// tag.set_attr("id", "title");
     /// tag.set_attr("class", "section1");
     ///
@@ -153,62 +153,62 @@ impl Tag {
     pub fn is_terminator(&self) -> bool {
         self.terminator
     }
-}
 
-/// Returns true if p is a sufficient condition for q.
-/// `p => q`
-///
-/// # Examples
-/// ```rust
-/// use std::collections::HashMap;
-/// use parsercher::dom::Tag;
-///
-/// let mut p = Tag::new("h1".to_string());
-/// p.set_attr("class", "target");
-///
-/// let mut q = Tag::new("h1".to_string());
-/// q.set_attr("id", "q");
-/// q.set_attr("class", "target");
-///
-/// assert_eq!(parsercher::satisfy_sufficient_condition(&p, &q), true);
-///
-/// let mut q = Tag::new("h1".to_string());
-/// q.set_attr("id", "q");
-///
-/// assert_eq!(parsercher::satisfy_sufficient_condition(&p, &q), false);
-/// ```
-pub fn satisfy_sufficient_condition(p: &Tag, q: &Tag) -> bool {
-    let mut satisfied = false;
-    // name
-    if q.get_name() == p.get_name() {
-        satisfied = true;
-    }
+    /// Returns true if p is a sufficient condition for q.
+    /// `p => q`
+    ///
+    /// # Examples
+    /// ```rust
+    /// use std::collections::HashMap;
+    /// use parsercher::dom::Tag;
+    ///
+    /// let mut p = Tag::new("h1");
+    /// p.set_attr("class", "target");
+    ///
+    /// let mut q = Tag::new("h1");
+    /// q.set_attr("id", "q");
+    /// q.set_attr("class", "target");
+    ///
+    /// assert_eq!(Tag::p_implies_q(&p, &q), true);
+    ///
+    /// let mut q = Tag::new("h1");
+    /// q.set_attr("id", "q");
+    ///
+    /// assert_eq!(Tag::p_implies_q(&p, &q), false);
+    /// ```
+    pub fn p_implies_q(p: &Tag, q: &Tag) -> bool {
+        let mut satisfied = false;
+        // name
+        if q.get_name() == p.get_name() {
+            satisfied = true;
+        }
 
-    // attr
-    if satisfied {
-        if let Some(p_attrs) = p.get_attrs() {
-            match q.get_attrs() {
-                Some(q_attrs) => {
-                    for (p_key, p_value) in p_attrs.iter() {
-                        match q_attrs.get(p_key) {
-                            Some(q_value) => {
-                                if p_value != "" && p_value != q_value {
+        // attr
+        if satisfied {
+            if let Some(p_attrs) = p.get_attrs() {
+                match q.get_attrs() {
+                    Some(q_attrs) => {
+                        for (p_key, p_value) in p_attrs.iter() {
+                            match q_attrs.get(p_key) {
+                                Some(q_value) => {
+                                    if p_value != "" && p_value != q_value {
+                                        satisfied = false;
+                                        break;
+                                    }
+                                }
+                                None => {
                                     satisfied = false;
                                     break;
                                 }
                             }
-                            None => {
-                                satisfied = false;
-                                break;
-                            }
                         }
                     }
+                    None => satisfied = false,
                 }
-                None => satisfied = false,
             }
         }
+        satisfied
     }
-    satisfied
 }
 
 #[cfg(test)]
@@ -217,13 +217,13 @@ mod tests {
 
     #[test]
     fn get_empty_attr() {
-        let tag = Tag::new("h1".to_string());
+        let tag = Tag::new("h1");
         assert_eq!(None, tag.get_attr("class"));
     }
 
     #[test]
     fn set_attr() {
-        let mut tag = Tag::new("h1".to_string());
+        let mut tag = Tag::new("h1");
         tag.set_attr("class", "section1");
         assert_eq!(Some("section1".to_string()), tag.get_attr("class"));
         assert_eq!(None, tag.get_attr("id"));
@@ -231,24 +231,52 @@ mod tests {
 
     #[test]
     fn sufficient_condition() {
-        let mut p = Tag::new("h1".to_string());
+        let mut p = Tag::new("h1");
         p.set_attr("class", "target");
 
-        let mut q = Tag::new("h1".to_string());
+        let mut q = Tag::new("h1");
         q.set_attr("id", "q");
         q.set_attr("class", "target");
 
-        assert_eq!(satisfy_sufficient_condition(&p, &q), true);
+        assert_eq!(Tag::p_implies_q(&p, &q), true);
     }
 
     #[test]
     fn not_sufficient_condition() {
-        let mut p = Tag::new("h1".to_string());
+        let mut p = Tag::new("h1");
         p.set_attr("class", "target");
 
-        let mut q = Tag::new("h1".to_string());
+        let mut q = Tag::new("h1");
         q.set_attr("id", "q");
 
-        assert_eq!(satisfy_sufficient_condition(&p, &q), false);
+        assert_eq!(Tag::p_implies_q(&p, &q), false);
+    }
+
+    #[test]
+    fn eq_test() {
+        let mut a = Tag::new("h1");
+        a.set_attr("id", "idA");
+        a.set_attr("class", "classA");
+
+        let mut b = Tag::new("h1");
+        b.set_attr("id", "idA");
+        b.set_attr("class", "classA");
+
+        assert_eq!(a == b, true);
+        assert_eq!(a != b, false);
+    }
+
+    #[test]
+    fn ne_test() {
+        let mut a = Tag::new("h1");
+        a.set_attr("id", "idA");
+        a.set_attr("class", "classA");
+
+        let mut b = Tag::new("h2");
+        b.set_attr("id", "idB");
+        b.set_attr("class", "classB");
+
+        assert_eq!(a == b, false);
+        assert_eq!(a != b, true);
     }
 }
